@@ -4,6 +4,8 @@
 #'
 #' @param x feature matrix.
 #' @param y a factor class variable with two classes.
+#' @param n_needed vector of desired number of synthetic samples for each class.
+#' A vector of integers for each class. Default is NULL meaning full balance.
 #'
 #' @details
 #' Random Oversampling (ROS) is a method of copying and pasting of positive
@@ -35,7 +37,7 @@
 #' @export
 
 
-ROS <- function(x, y) {
+ROS <- function(x, y, n_needed = NULL) {
 
   if (is.data.frame(x)) {
     x <- as.matrix(x)
@@ -58,7 +60,12 @@ ROS <- function(x, y) {
   n_classes <- sapply(class_names, function(m) sum(y == m))
   k_class <- length(class_names)
   n_classes_max <- max(n_classes)
-  n_needed <- n_classes_max - n_classes
+  if (is.null(n_needed)) {
+    n_needed <- max(n_classes) - n_classes
+  }
+  if (length(n_needed) != k_class) {
+    stop("n_needed must be an integer vector matching the number of classes.")
+  }
   x_classes <- lapply(class_names, function(m) x[y == m,, drop = FALSE])
   x_syn_list <- list()
 
